@@ -178,7 +178,7 @@ import traceback
 from musamusa_errors.error_messages import ListOfErrorMessages, MusTextFileError, MusTextFileWarning
 from musamusa_fal.fal import FileAndLine
 from musamusa_etr.utils import path_of
-from musamusa_etr.etr_line import ETRLine
+from musamusa_etr.etr_line import ETRLine, TLFF
 from musamusa_etr.etr_line import ETRLINE_INTTYPE__NORMAL, ETRLINE_INTTYPE__COMMENTEMPTY
 from musamusa_etr.etr_line import ETRLINE_INTTYPE__PARSINGTOOLSDEFINITION
 from musamusa_etr.etr_line import ETRLINE_INTTYPE__VARIABLE, ETRLINE_INTTYPE__TAG
@@ -771,6 +771,7 @@ class ETR:
         # ---- returned value -------------------------------------------------
         # (pimydoc)flags
         # ⋅Flags are sorted alphabetically via the sorted() function.
+        # ⋅In fine, flags must be a tuple of strings : they can't be None.
         return (line.replace(self.parsingtools["tobecontinued_char"],
                              ""),
                 tuple(sorted(read_flags)))
@@ -790,7 +791,10 @@ class ETR:
             o  (str)line        : the line where the tag could be read.
             o  (list of FAL)fals: the FALs objects associated to the <line>.
 
-            RETURNED VALUE: (bool)has a tag been read in <line> ?
+            RETURNED VALUE: ((bool)has a tag been read in <line> ?,
+                             (str)tag name,
+                             (str)tag value,
+                            )
         """
         res = False
 
@@ -876,7 +880,9 @@ class ETR:
                 error.fals = fals
                 self.errors.append(error)
 
-        return res
+            return res, tag.group("name"), tag.group("value")
+
+        return res, None, None
 
     def _parse_read_line__variable(self,
                                    line: str,
@@ -898,7 +904,9 @@ class ETR:
             o  (str)line        : the line where the variable could be read.
             o  (list of FAL)fals: the FALs objects associated to the <line>.
 
-            RETURNED VALUE: (bool)has a variable been read in <line> ?
+            RETURNED VALUE: ((bool)has a variable been read in <line> ?,
+                             (str)variable name,
+                             (str)variable value)
         """
         res = False
 
@@ -923,7 +931,9 @@ class ETR:
 
             self.read_variables[variable.group("name")] = (variable.group("value"), fals)
 
-        return res
+            return res, variable.group("name"), variable.group("value")
+
+        return res, None, None
 
     def _read(self,
               source_filename: str):
@@ -969,7 +979,7 @@ class ETR:
                 f"Cyclic import in '{source_filename}' ." \
                 f"Parent files already read are f{self._nested_files}"
             self.errors.append(error)
-            yield from self._read_action(line_inttype=ETRLINE_INTTYPE__NONE)
+            yield from self._read_action(TLFF(line_inttype=ETRLINE_INTTYPE__NONE))
             return
 
         self._nested_files.append(source_filename)
@@ -982,7 +992,7 @@ class ETR:
             error.msg = f"({error.msgid}) " \
                 f"Missing ETR MusaMusa text file '{source_filename}' ."
             self.errors.append(error)
-            yield from self._read_action(line_inttype=ETRLINE_INTTYPE__NONE)
+            yield from self._read_action(TLFF(line_inttype=ETRLINE_INTTYPE__NONE))
             return
 
         # main loop:
@@ -1000,7 +1010,7 @@ class ETR:
                 f"Python error is (KeyError)'{traceback.format_exc()}'."
             self.errors.append(error)
 
-            yield from self._read_action(line_inttype=ETRLINE_INTTYPE__NONE)
+            yield from self._read_action(TLFF(line_inttype=ETRLINE_INTTYPE__NONE))
 
         self._nested_files.pop()
 
@@ -1035,6 +1045,7 @@ class ETR:
         #
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-statements
+        # pylint: disable=too-many-locals
         ptoo = self.parsingtools
 
         with open(source_filename,
@@ -1057,11 +1068,11 @@ class ETR:
 
                 # empty lines are ignored and comment lines are ignored:
                 if _line.strip() == "" or _line.startswith(ptoo["comment_lineprefix"]):
-                    yield from self._read_action(line_inttype=ETRLINE_INTTYPE__COMMENTEMPTY,
-                                                 fals=(FileAndLine(filename=source_filename,
-                                                                   lineindex=line_index+1),),
-                                                 flags=None,
-                                                 line=_line.strip())
+                    yield from self._read_action(TLFF(line_inttype=ETRLINE_INTTYPE__COMMENTEMPTY,
+                                                      fals=(FileAndLine(filename=source_filename,
+                                                                        lineindex=line_index+1),),
+                                                      flags=None,
+                                                      line=_line.strip()))
                     continue
 
                 # ---- is there a new "parsingtools_definition" ? ---------
@@ -1072,35 +1083,47 @@ class ETR:
                                                fals=(FileAndLine(filename=source_filename,
                                                                  lineindex=line_index+1),))
                     yield from self._read_action(
-                        line_inttype=ETRLINE_INTTYPE__PARSINGTOOLSDEFINITION,
-                        fals=(FileAndLine(filename=source_filename,
-                                          lineindex=line_index+1),),
-                        flags=None,
-                        line=_line.strip())
+                        TLFF(line_inttype=ETRLINE_INTTYPE__PARSINGTOOLSDEFINITION,
+                             fals=(FileAndLine(filename=source_filename,
+                                               lineindex=line_index+1),),
+                             flags=None,
+                             line=_line.strip()))
                     continue
 
                 # ---- is there a new variable definition ? ---------------------------
-                if self._parse_read_line__variable(line=_line,
-                                                   fals=(FileAndLine(filename=source_filename,
-                                                                     lineindex=line_index+1),)):
+                is_a_variable, variable_name, variable_value = \
+                    self._parse_read_line__variable(
+                        line=_line,
+                        fals=(FileAndLine(filename=source_filename,
+                                          lineindex=line_index+1),))
+                if is_a_variable:
                     # yes, it was a variable definition.
-                    yield from self._read_action(line_inttype=ETRLINE_INTTYPE__VARIABLE,
-                                                 fals=(FileAndLine(filename=source_filename,
-                                                                   lineindex=line_index+1),),
-                                                 flags=None,
-                                                 line=_line.strip())
+                    yield from self._read_action(
+                        TLFF(line_inttype=ETRLINE_INTTYPE__VARIABLE,
+                             fals=(FileAndLine(filename=source_filename,
+                                               lineindex=line_index+1),),
+                             flags=None,
+                             line=_line.strip()),
+                        details={"variable_name": variable_name,
+                                 "variable_value": variable_value})
                     continue
 
                 # ---- is there a new tag definition ? --------------------------------
-                if self._parse_read_line__tag(line=_line,
-                                              fals=(FileAndLine(filename=source_filename,
-                                                                lineindex=line_index+1),)):
+                is_a_tag, tag_name, tag_value = \
+                    self._parse_read_line__tag(line=_line,
+                                               fals=(FileAndLine(filename=source_filename,
+                                                                 lineindex=line_index+1),))
+
+                if is_a_tag:
                     # yes, it was a variable definition.
-                    yield from self._read_action(line_inttype=ETRLINE_INTTYPE__TAG,
-                                                 fals=(FileAndLine(filename=source_filename,
-                                                                   lineindex=line_index+1),),
-                                                 flags=None,
-                                                 line=_line.strip())
+                    yield from self._read_action(
+                        TLFF(line_inttype=ETRLINE_INTTYPE__TAG,
+                             fals=(FileAndLine(filename=source_filename,
+                                               lineindex=line_index+1),),
+                             flags=None,
+                             line=_line.strip()),
+                        details={"tag_name": tag_name,
+                                 "tag_value": tag_value})
                     continue
 
                 # with the "left-spaces syntax" we have to preserve left-spaces string,
@@ -1158,9 +1181,11 @@ class ETR:
                         next_line.line, next_line.flags = self._parse_read_line(
                             line=next_line.line)
                         if next_line.line:  # may be None (see self._parse_read_line())
-                            yield from self._read_action(fals=tuple(next_line.fals),
-                                                         flags=next_line.flags,
-                                                         line=next_line.line)
+                            yield from self._read_action(
+                                TLFF(line_inttype=ETRLINE_INTTYPE__NORMAL,
+                                     fals=tuple(next_line.fals),
+                                     flags=next_line.flags,
+                                     line=next_line.line))
                         next_line = None
 
                     # recursive call to self._read():
@@ -1183,9 +1208,11 @@ class ETR:
                         next_line.line, next_line.flags = self._parse_read_line(
                             line=next_line.line)
                         if next_line.line:  # may be None (see self._parse_read_line())
-                            yield from self._read_action(fals=tuple(next_line.fals),
-                                                         flags=next_line.flags,
-                                                         line=next_line.line)
+                            yield from self._read_action(
+                                TLFF(line_inttype=ETRLINE_INTTYPE__NORMAL,
+                                     fals=tuple(next_line.fals),
+                                     flags=next_line.flags,
+                                     line=next_line.line))
 
                     # we have a new <next_line>:
                     next_line = ETRLine(fals=[FileAndLine(filename=source_filename,
@@ -1201,25 +1228,15 @@ class ETR:
             next_line.line, next_line.flags = self._parse_read_line(
                 line=next_line.line)
             if next_line.line:  # may be None (see self._parse_read_line())
-                yield from self._read_action(fals=tuple(next_line.fals),
-                                             flags=next_line.flags,
-                                             line=next_line.line)
-
-                # we have a new <next_line>:
-                _line2, _flags2 = self._parse_read_line(
-                    line=line)
-                if _line2:
-                    next_line = ETRLine(fals=(FileAndLine(filename=source_filename,
-                                                          lineindex=line_index+1), ),
-                                        line=_line2,
-                                        flags=_flags2,
-                                        line_inttype=ETRLINE_INTTYPE__NORMAL)
+                yield from self._read_action(
+                    TLFF(line_inttype=ETRLINE_INTTYPE__NORMAL,
+                         fals=tuple(next_line.fals),
+                         flags=next_line.flags,
+                         line=next_line.line))
 
     def _read_action(self,
-                     line_inttype=ETRLINE_INTTYPE__NORMAL,
-                     fals=None,
-                     flags=None,
-                     line=None):
+                     tlff,
+                     details=None):
         """
             ETR._read_action()
 
@@ -1232,52 +1249,62 @@ class ETR:
             ___________________________________________________________________
 
             ARGUMENTS:
-            o  (int)line_inttype     : (int) ETRLINE_xxx constant
-            o  (None|list of FAL)fals: the FALs objects associated to the <line>.
-            o  (None|tuple)flags     : None or a list of str.
-            o  (None|str)line        : the line that has been read
+            o  tlff                  : a TLFF object (line_inttype, line, flags, fals)
+
+            o  (None|dict)details    : (str:str) details about the read line.
+               (pimydoc)ETR._read_action():details
+               ⋅<details> is a dict (str:str) allowing to store the last read variable or
+               ⋅the last read tag.
+               ⋅
+               ⋅Known keys are:
+               ⋅    o  "variable_name"
+               ⋅    o  "variable_value"
+               ⋅    o  "tag_name"
+               ⋅    o  "tag_value"
+               ⋅
+               ⋅By example, details["variable_name"] and details["variable_value"] will
+               ⋅give the last pair of read variable name/value.
 
             This method may yield ETRLine objects.
         """
-        if line_inttype == ETRLINE_INTTYPE__NORMAL:
+        if tlff.line_inttype == ETRLINE_INTTYPE__NORMAL:
             if self.options["read line:normal:event"]:
-                self.options["read line:normal:event"](fals,
-                                                       flags,
-                                                       line,
-                                                       line_inttype=line_inttype)
+                self.options["read line:normal:event"](tlff.fals,
+                                                       tlff.flags,
+                                                       tlff.line,
+                                                       tlff.line_inttype,
+                                                       details)
 
             if self.options["read line:normal:yield"]:
                 yield ETRLine(
-                    fals=fals,
-                    flags=flags,
-                    line=line,
-                    line_inttype=line_inttype)
+                    fals=tlff.fals,
+                    flags=tlff.flags,
+                    line=tlff.line,
+                    line_inttype=tlff.line_inttype)
 
-        elif line_inttype == ETRLINE_INTTYPE__COMMENTEMPTY:
+        elif tlff.line_inttype == ETRLINE_INTTYPE__COMMENTEMPTY:
             if self.options["read line:empty line/comment:event"]:
-                self.options["read line:empty line/comment:event"](fals,
-                                                                   flags,
-                                                                   line)
+                self.options["read line:empty line/comment:event"](tlff.fals,
+                                                                   tlff.flags,
+                                                                   tlff.line,
+                                                                   tlff.line_inttype,
+                                                                   details)
 
             if self.options["read line:empty line/comment:yield"]:
                 yield ETRLine(
-                    fals=fals,
-                    flags=flags,
-                    line=line,
-                    line_inttype=line_inttype)
+                    fals=tlff.fals,
+                    flags=tlff.flags,
+                    line=tlff.line,
+                    line_inttype=tlff.line_inttype)
 
         else:
             # rarest cases:
-            yield from self._read_action2(line_inttype,
-                                          fals,
-                                          flags,
-                                          line)
+            yield from self._read_action2(tlff,
+                                          details)
 
     def _read_action2(self,
-                      line_inttype=ETRLINE_INTTYPE__NORMAL,
-                      fals=None,
-                      flags=None,
-                      line=None):
+                      tlff,
+                      details=None):
         """
             ETR._read_action2()
 
@@ -1291,58 +1318,77 @@ class ETR:
             on the content of .options["read line:xxx")
             ___________________________________________________________________
 
-            ARGUMENTS:
-            o  (int)line_inttype     : (int) ETRLINE_xxx constant
-            o  (None|list of FAL)fals: the FALs objects associated to the <line>.
-            o  (None|tuple)flags     : None or a list of str.
-            o  (None|str)line        : the line that has been read
+            ARGUMENTS: TODO
+            o  tlff                  : a TLFF object (line_inttype, line, flags, fals)
+
+            o  (None|dict)details    : (str:str) details about the read line.
+               (pimydoc)ETR._read_action():details
+               ⋅<details> is a dict (str:str) allowing to store the last read variable or
+               ⋅the last read tag.
+               ⋅
+               ⋅Known keys are:
+               ⋅    o  "variable_name"
+               ⋅    o  "variable_value"
+               ⋅    o  "tag_name"
+               ⋅    o  "tag_value"
+               ⋅
+               ⋅By example, details["variable_name"] and details["variable_value"] will
+               ⋅give the last pair of read variable name/value.
 
             This method may yield ETRLine objects.
         """
-        if line_inttype == ETRLINE_INTTYPE__PARSINGTOOLSDEFINITION:
+        if tlff.line_inttype == ETRLINE_INTTYPE__PARSINGTOOLSDEFINITION:
             if self.options["read line:parsingtools_definition:event"]:
-                self.options["read line:parsingtools_definition:event"](fals,
-                                                                        flags,
-                                                                        line)
+                self.options["read line:parsingtools_definition:event"](tlff.fals,
+                                                                        tlff.flags,
+                                                                        tlff.line,
+                                                                        tlff.line_inttype,
+                                                                        details)
 
             if self.options["read line:parsingtools_definition:yield"]:
                 yield ETRLine(
-                    fals=fals,
-                    flags=flags,
-                    line=line,
-                    line_inttype=line_inttype)
+                    fals=tlff.fals,
+                    flags=tlff.flags,
+                    line=tlff.line,
+                    line_inttype=tlff.line_inttype)
 
-        elif line_inttype == ETRLINE_INTTYPE__VARIABLE:
+        elif tlff.line_inttype == ETRLINE_INTTYPE__VARIABLE:
             if self.options["read line:variable:event"]:
-                self.options["read line:variable:event"](fals,
-                                                         flags,
-                                                         line)
+                self.options["read line:variable:event"](tlff.fals,
+                                                         tlff.flags,
+                                                         tlff.line,
+                                                         tlff.line_inttype,
+                                                         details)
 
             if self.options["read line:variable:yield"]:
                 yield ETRLine(
-                    fals=fals,
-                    flags=flags,
-                    line=line,
-                    line_inttype=line_inttype)
+                    fals=tlff.fals,
+                    flags=tlff.flags,
+                    line=tlff.line,
+                    line_inttype=tlff.line_inttype)
 
-        elif line_inttype == ETRLINE_INTTYPE__TAG:
+        elif tlff.line_inttype == ETRLINE_INTTYPE__TAG:
             if self.options["read line:tag:event"]:
-                self.options["read line:tag:event"](fals,
-                                                    flags,
-                                                    line)
+                self.options["read line:tag:event"](tlff.fals,
+                                                    tlff.flags,
+                                                    tlff.line,
+                                                    tlff.line_inttype,
+                                                    details)
 
             if self.options["read line:tag:yield"]:
                 yield ETRLine(
-                    fals=fals,
-                    flags=flags,
-                    line=line,
-                    line_inttype=line_inttype)
+                    fals=tlff.fals,
+                    flags=tlff.flags,
+                    line=tlff.line,
+                    line_inttype=tlff.line_inttype)
 
-        elif line_inttype is ETRLINE_INTTYPE__NONE:
+        elif tlff.line_inttype is ETRLINE_INTTYPE__NONE:
             if self.options["read line:None:event"]:
-                self.options["read line:None:event"](fals,
-                                                     flags,
-                                                     line)
+                self.options["read line:None:event"](tlff.fals,
+                                                     tlff.flags,
+                                                     tlff.line,
+                                                     tlff.line_inttype,
+                                                     details)
 
             # These lines are superfluous as the value None will be yielded
             # by the method anyway, at the end of the method:
